@@ -32,10 +32,14 @@
 AsnOid atmfPortIndex;
 AsnOid atmfMyIpNmAddress;
 AsnOid atmfNetPrefixStatus;
+AsnOid atmfAtmLayerMaxVpiBits;
+AsnOid atmfAtmLayerMaxVciBits;
 AsnOid atmfAtmLayerUniVersion;
 
 AsnInt atmfPortIndexValue;
 IpAddress atmfMyIpNmAddressValue;
+AsnInt atmfAtmLayerMaxVpiBitsValue;
+AsnInt atmfAtmLayerMaxVciBitsValue;
 AsnInt atmfAtmLayerUniVersionValue;
 
 AsnOid *accessNetPrefix(void);
