@@ -1,14 +1,13 @@
 /* text2atm.c - Converts textual representation of ATM address to binary
 		encoding */
 
-/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1998 by Werner Almesberger, EPFL-LRC/ICA */
 
 
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 #include <limits.h>
-#include <linux/atm.h>
 
 #include "atm.h"
 #include "atmsap.h"
