@@ -1,7 +1,7 @@
 /* text2qos.c - Converts textual representation of QOS parameters to binary
 		encoding */
 
-/* Written 1996,1997 by Werner Almesberger, EPFL-LRC */
+/* Written 1996-1998 by Werner Almesberger, EPFL-LRC/ICA */
 
 
 #include <stdlib.h>
@@ -9,7 +9,6 @@
 #include <string.h>
 #include <ctype.h>
 #include <limits.h>
-#include <linux/atm.h>
 
 #include "atm.h"
 
