@@ -1,11 +1,14 @@
 /* pdu.h - SSCOP (Q.2110) PDU reader */
 
-/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1998 by Werner Almesberger, EPFL-LRC/ICA */
 
 
 #ifndef PDU_H
 #define PDU_H
 
+#include <stdint.h>
+
+
 /* SSCOP PDU types, Q.2110 section 7.1 */
 
 #define SSCOP_BGN	 1 /* Request Initialization */
@@ -38,7 +41,7 @@
 /* Work around a buggy ntohl prototype in some early version of glibc on 64 bit
    platforms. */
 
-#define _ntohl(l) (ntohl((__u32) (l)) & 0xffffffffU)
+#define _ntohl(l) (ntohl((uint32_t) (l)) & 0xffffffffU)
 
 
 extern const char *pdu_name[];
