@@ -45,9 +45,16 @@ static int send_setup(SOCKET *sock)
     q_create(&dsc,q_buffer,MAX_Q_MSG);
     local = NULL;
     error = 0;
-    if (atmsvc_addr_in_use(sock->local)) local = &sock->local;
+    if (atmsvc_addr_in_use(sock->local)) {
+	sock->sig = route_local(&sock->local);
+	if (sock->sig) local = &sock->local;
+	else {
+	    error = -EADDRNOTAVAIL;
+	    diag(COMPONENT,DIAG_WARN,"local address no longer available");
+	}
+    }
     else {
-	sock->sig = route_call(&sock->remote);
+	sock->sig = route_remote(&sock->remote);
 	if (!sock->sig) error = -EHOSTUNREACH;
 	else {
 	    local = get_local(sock->sig->signaling_pvc.sap_addr.itf);
@@ -75,7 +82,7 @@ static int send_setup(SOCKET *sock)
     q_assign(&dsc,QF_msg_type,ATM_MSG_SETUP);
     q_assign(&dsc,QF_aal_type,5); /* AAL 5 */
 #if defined(UNI30) || defined(DYNAMIC_UNI)
-    if (sock->sig->uni == S_UNI30)
+    if (!error && sock->sig->uni == S_UNI30)
 	q_assign(&dsc,QF_aal_mode,1); /* Message mode - LANE seems to really
 					 want this */
 #endif
@@ -85,18 +92,18 @@ static int send_setup(SOCKET *sock)
 	  sock->sig->uni);
     q_assign(&dsc,QF_bearer_class,16); /* BCOB-X */
 #if defined(UNI30) || defined(ALLOW_UNI30) || defined(DYNAMIC_UNI)
-    if (sock->sig->uni & S_UNI30)
+    if (!error && sock->sig->uni & S_UNI30)
 	q_assign(&dsc,QF_trans_cap,ATM_TC_VBR_NRT_R00);
 	  /* force presence - UNI 3.0 wants this */
 #endif
     q_assign(&dsc,QF_upcc,0); /* p2p */
 #if !defined(UNI30) || defined(DYNAMIC_UNI)
-    if (sock->sig->uni != S_UNI30)
+    if (!errror && sock->sig->uni != S_UNI30)
 	q_assign(&dsc,QF_qos_cs,Q2931_CS_ITU);
 #endif
     q_assign(&dsc,QF_qos_fw,0); /* QOS 0 */
     q_assign(&dsc,QF_qos_bw,0);
-    if (sock->sig->mode != sm_user) {
+    if (!error && sock->sig->mode != sm_user) {
 	if (!atmpvc_addr_in_use(sock->pvc)) {
 	    int vpci,vci;
 
@@ -203,23 +210,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		send_kernel(msg->vcc,0,as_okay,0,NULL,NULL,
 		      &local_addr[0].addr,NULL,NULL);
 	    else {
-		LOCAL_ADDR *walk;
-
-		for (walk = local_addr; walk->state != ls_unused; walk++)
-		    if (walk->state == ls_same &&
-		      atm_equal((struct sockaddr *) &walk->addr,
-		      (struct sockaddr *) &msg->svc,(ATM_ESA_LEN-1)*8,
-		      AXE_WILDCARD)) break;
-		    if(*msg->svc.sas_addr.pub)
-			diag(COMPONENT,DIAG_DEBUG,"binding to E.164 address "
-			  "%s\n",msg->svc.sas_addr.pub);
-		if (walk->state == ls_unused) {
-		    SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
-		    return;
-		}
-		sock->sig = get_sig_entity(walk->itf);
-		if (!sock->sig) {
-		    diag(COMPONENT,DIAG_ERROR,"signaling entity lookup failed");
+		if (!route_local(&msg->svc)) {
 		    SEND_ERROR(msg->vcc,-EADDRNOTAVAIL);
 		    return;
 		}
@@ -233,6 +224,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		return;
 	    }
 	    sock = new_sock(msg->vcc);
+	    sock->local = msg->local;
 	    sock->remote = msg->svc;
 	    sock->qos = msg->qos;
 	    sock->sap = msg->sap;
