@@ -17,6 +17,7 @@
 #include <q.out.h>
 
 #include "common.h"
+#include "proto.h"
 #include "sap.h"
 
 
@@ -129,7 +130,7 @@ static int encode_blli(Q_DSC *dsc,const struct atm_blli *blli)
 
 
 int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *addr,
-  const struct atm_sap *sap,const struct atm_qos *qos)
+  const struct atm_sap *sap,const struct atm_qos *qos,int uni)
 {
     int error,pcr;
 
@@ -141,9 +142,12 @@ int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *addr,
 	else return -EDESTADDRREQ;
     if (qos->txtp.traffic_class == ATM_UBR || qos->rxtp.traffic_class ==
       ATM_UBR) q_assign(dsc,QF_best_effort,0);
-#ifdef UNI40
+#if defined(UNI40) || defined(DYNAMIC_UNI)
     if (qos->txtp.traffic_class == ATM_CBR || qos->rxtp.traffic_class ==
-      ATM_CBR) q_assign(dsc,QF_trans_cap,ATM_TC_CBR);
+      ATM_CBR) {
+	if (!(uni & S_UNI40)) return -EINVAL;
+	q_assign(dsc,QF_trans_cap,ATM_TC_CBR);
+    }
 #endif
     switch (qos->txtp.traffic_class) {
 	case ATM_NONE:
@@ -195,8 +199,9 @@ int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *addr,
             case ATM_HL_USER:
                 q_write(dsc,QF_user_hli,sap->bhli.hl_info,sap->bhli.hl_length);
                 break;
-#ifdef UNI30
+#if defined(UNI30) || defined(DYNAMIC_UNI)
             case ATM_HL_HLP:
+		if (!(uni & S_UNI30)) return -EINVAL;
                 q_write(dsc,QF_hlp,sap->bhli.hl_info,4);
                 break;
 #endif
@@ -256,8 +261,8 @@ static void decode_blli(Q_DSC *dsc,struct atm_blli *blli)
 }
 
 
-void sap_decode(Q_DSC *dsc,struct sockaddr_atmsvc *addr,struct atm_sap *sap,
-  struct atm_qos *qos)
+unsigned int sap_decode(Q_DSC *dsc,struct sockaddr_atmsvc *addr,
+  struct atm_sap *sap,struct atm_qos *qos,int uni)
 {
     memset(addr,0,sizeof(*addr));
     memset(sap,0,sizeof(*sap));
@@ -269,10 +274,22 @@ void sap_decode(Q_DSC *dsc,struct sockaddr_atmsvc *addr,struct atm_sap *sap,
     else if (q_present(dsc,QF_cdpn_esa))
 	    (void) q_read(dsc,QF_cdpn_esa,(void *) &addr->sas_addr.prv,
 	      ATM_ESA_LEN);
+    qos->aal = ATM_AAL5; /* default and only choice */
     if (q_present(dsc,QF_aal_type))
-	if (q_fetch(dsc,QF_aal_type) != 5)
+	if (q_fetch(dsc,QF_aal_type) != 5) {
 	    diag(COMPONENT,DIAG_ERROR,"AAL type %d requested",
 	      q_fetch(dsc,QF_aal_type));
+#ifdef DYNAMIC_UNI
+	    return IE_PROBLEM(
+	      uni & S_UNI30 ? ATM_CV_AAL_UNSUPP_OLD : ATM_CV_AAL_UNSUPP_NEW,0);
+#else
+#ifdef UNI30
+	    return IE_PROBLEM(ATM_CV_AAL_UNSUPP_OLD,0);
+#else
+	    return IE_PROBLEM(ATM_CV_AAL_UNSUPP_NEW,0);
+#endif /* UNI30 */
+#endif /* DYNAMIC_UNI */
+	}
     if (q_present(dsc,QF_best_effort)) {
 	qos->txtp.traffic_class = qos->rxtp.traffic_class = ATM_UBR;
 	diag(COMPONENT,DIAG_DEBUG,"UBR");
@@ -312,28 +329,34 @@ void sap_decode(Q_DSC *dsc,struct sockaddr_atmsvc *addr,struct atm_sap *sap,
 		sap->bhli.hl_length = q_length(dsc,QF_user_hli);
 		q_read(dsc,QF_user_hli,sap->bhli.hl_info,sap->bhli.hl_length);
 		break;
-#ifdef UNI30
-	    case ATM_HL_HLP:
-		sap->bhli.hl_length = 4;
-		q_read(dsc,QF_hlp,sap->bhli.hl_info,4);
-		break;
-#endif
 	    case ATM_HL_VENDOR:
 		sap->bhli.hl_length = 7;
 		q_read(dsc,QF_hli_oui,sap->bhli.hl_info,3);
 		q_read(dsc,QF_app_id,sap->bhli.hl_info+3,4);
 		break;
+#if defined(UNI30) || defined(DYNAMIC_UNI)
+	    case ATM_HL_HLP:
+		if (!(uni & S_UNI30)) {
+		    sap->bhli.hl_length = 4;
+		    q_read(dsc,QF_hlp,sap->bhli.hl_info,4);
+		    break;
+		}
+		/* fall through */
+#endif
 	    default:
-		diag(COMPONENT,DIAG_FATAL,"unrecognized hl_type");
+		diag(COMPONENT,DIAG_ERROR,"unrecognized hl_type %d",
+		  sap->bhli.hl_type);
+		return IE_PROBLEM(ATM_CV_INVALID_IE,ATM_IE_BHLI);
 	}
     }
-    if (!q_present(dsc,QG_blli1)) return;
+    if (!q_present(dsc,QG_blli1)) return 0;
     q_instance(dsc,QG_blli1);
     decode_blli(dsc,sap->blli);
-    if (!q_present(dsc,QG_blli2)) return;
+    if (!q_present(dsc,QG_blli2)) return 0;
     q_instance(dsc,QG_blli2);
     decode_blli(dsc,sap->blli+1);
-    if (!q_present(dsc,QG_blli3)) return;
+    if (!q_present(dsc,QG_blli3)) return 0;
     q_instance(dsc,QG_blli3);
     decode_blli(dsc,sap->blli+2);
+    return 0;
 }
